@@ -5,6 +5,22 @@ import ee
 from . import calcs
 
 
+def lazy_property(fn):
+    """Decorator that makes a property lazy-evaluated
+
+    https://stevenloria.com/lazy-properties/
+    """
+    attr_name = '_lazy_' + fn.__name__
+
+    @property
+    def _lazy_property(self):
+        if not hasattr(self, attr_name):
+            setattr(self, attr_name, fn(self))
+        return getattr(self, attr_name)
+
+    return _lazy_property
+
+
 class Daily():
     """"""
 
@@ -161,12 +177,13 @@ class Daily():
 
         """
         if surface.lower() in ['alfalfa', 'etr', 'tall']:
-            return self.etr()
+            return self.etr
         elif surface.lower() in ['grass', 'eto', 'short']:
-            return self.eto()
+            return self.eto
         else:
             raise ValueError('unsupported surface type: {}'.format(surface))
 
+    @lazy_property
     def eto(self):
         """Short (grass) reference surface"""
         self.cn = 900
@@ -174,6 +191,7 @@ class Daily():
         return ee.Image(self._etsz().rename(['eto'])
             .set('system:time_start', self.time_start))
 
+    @lazy_property
     def etr(self):
         """Tall (alfalfa) reference surface"""
         self.cn = 1600
@@ -203,6 +221,33 @@ class Daily():
         #      'psy': self.psy, 'rn': self.rn, 'tmean': self.tmean,
         #      'u2': self.u2, 'vpd': self.vpd})
 
+    @lazy_property
+    def etw(self):
+        """Priestley-Taylor evaporation (alpha = 1.26)
+
+        Returns
+        -------
+        etw : ee.Image
+            Priestley-Taylor ET [mm].
+
+        References
+        ----------
+        https://wetlandscapes.github.io/blog/blog/penman-monteith-and-priestley-taylor/
+
+        """
+        return self.es_slope\
+            .expression(
+                '(alpha * es_slope * rn * 1000 / (2453 * (es_slope + psy)))',
+                {'es_slope': self.es_slope, 'rn': self.rn, 'psy': self.psy,
+                 'alpha': 1.26})\
+            .rename(['etw'])\
+            .set('system:time_start', self.time_start)
+        # return self.es_slope.multiply(self.rn)\
+        #     .divide((self.es_slope.add(self.psy)).multiply(2453))\
+        #     .multiply(1.26).multiply(1000)\
+        #     .rename(['etw'])\
+        #     .set('system:time_start', self.time_start)
+
     @classmethod
     def gridmet(cls, input_img, zw=None, elev=None, lat=None, method='asce',
                 rso_type=None):
@@ -279,7 +324,7 @@ class Daily():
 
     @classmethod
     def maca(cls, input_img, zw=None, elev=None, lat=None, method='asce',
-                rso_type=None):
+             rso_type=None):
         """Initialize daily RefET from a MACA image
 
         Parameters
@@ -397,8 +442,7 @@ class Daily():
 
         """
         input_coll = ee.ImageCollection(input_coll)
-        image_date = ee.Date(
-            ee.Image(input_coll.first()).get('system:time_start'))
+        image_date = ee.Date(ee.Image(input_coll.first()).get('system:time_start'))
 
         if zw is None:
             zw = ee.Number(10)
@@ -481,8 +525,7 @@ class Daily():
 
         """
         input_coll = ee.ImageCollection(input_coll)
-        image_date = ee.Date(
-            ee.Image(input_coll.first()).get('system:time_start'))
+        image_date = ee.Date(ee.Image(input_coll.first()).get('system:time_start'))
 
         cfsv2_crs = 'EPSG:4326'
         # Transform for 2011 to Present
@@ -537,3 +580,107 @@ class Daily():
             method=method,
             rso_type=rso_type,
         )
+
+    @classmethod
+    def rtma(cls, input_coll, rs=None, zw=None, elev=None, lat=None,
+             method='asce', rso_type=None):
+        """Initialize daily RefET from a hourly RTMA image collection
+
+        Parameters
+        ----------
+        input_coll : ee.ImageCollection
+            Collection of RTMA hourly images for a single day from the
+            collection NOAA/NWS/RTMA.
+        rs : ee.Image, str, optional
+            Incoming solar radiation [MJ m-2 day-1].  The GRIDMET image for the
+            concurrent day will be used if not set.
+        zw : ee.Number, optional
+            Wind speed height [m] (the default is 10).
+        elev : ee.Image or ee.Number, optional
+            Elevation image [m].  The RTMA elevation image
+            (projects/climate-engine/rtma/elevation) will be used if not set.
+        lat : ee.Image or ee.Number
+            Latitude image [degrees].  The latitude will be computed
+            dynamically using ee.Image.pixelLonLat() if not set.
+        method : {'asce' (default), 'refet'}, optional
+            Specifies which calculation method to use.
+            * 'asce' -- Calculations will follow ASCE-EWRI 2005.
+            * 'refet' -- Calculations will follow RefET software.
+        rso_type : {None (default), 'full' , 'simple'}, optional
+            Specifies which clear sky solar radiation (Rso) model to use.
+            * None -- Rso type will be determined from "method" parameter
+            * 'full' -- Full clear sky solar formulation
+            * 'simple' -- Simplified clear sky solar formulation
+
+        Notes
+        -----
+        Solar radiation is converted from W m-2 to MJ m-2 day-1.
+        Actual vapor pressure is computed from specific humidity and air
+            pressure (from elevation).
+
+        """
+        input_coll = ee.ImageCollection(input_coll)
+        start_date = ee.Date(ee.Image(input_coll.first()).get('system:time_start'))
+
+        # Parse the solar radiation input
+        if isinstance(rs, ee.Image):
+            pass
+        elif isinstance(rs, ee.Number) or isinstance(rs, float) or isinstance(rs, int):
+            rs = ee.Image.constant(rs)
+        elif rs is None or rs.upper() == 'GRIDMET':
+            rs_coll = ee.ImageCollection('IDAHO_EPSCOR/GRIDMET')\
+                .filterDate(start_date, start_date.advance(1, 'day'))\
+                .select(['srad'])
+            rs = ee.Image(rs_coll.first()).multiply(0.0864)
+        elif rs.upper() == 'NLDAS':
+            rs_coll = ee.ImageCollection('NASA/NLDAS/FORA0125_H002')\
+                .filterDate(start_date, start_date.advance(1, 'day'))\
+                .select(['shortwave_radiation'])
+            # TODO: Check this unit conversion
+            rs = ee.Image(rs_coll.sum()).multiply(0.0036)
+        else:
+            raise ValueError('Unsupported Rs input')
+        # CGM - For now I don't think it makes sense to support image collections
+        # elif isinstance(rs, ee.ImageCollection):
+        #     rs = ee.Image(rs.sum())
+        #     # rs = ee.Image(rs.mean())
+
+        if zw is None:
+            zw = ee.Number(10)
+        if elev is None:
+            elev = ee.Image('projects/earthengine-legacy/assets/'
+                            'projects/climate-engine/rtma/elevation')\
+                .rename(['elevation'])
+        if lat is None:
+            lat = ee.Image('projects/earthengine-legacy/assets/'
+                           'projects/climate-engine/rtma/elevation')\
+                .multiply(0).add(ee.Image.pixelLonLat().select('latitude'))\
+                .rename(['latitude'])
+
+        # DEADBEEF - Don't compute wind speed from the components since
+        #   a wind speed band is provided
+        # def wind_magnitude(input_img):
+        #     """Compute hourly wind magnitude from vectors"""
+        #     return ee.Image(input_img.select(['UGRD'])).pow(2)\
+        #         .add(ee.Image(input_img.select(['VGRD'])).pow(2))\
+        #         .sqrt().rename(['WIND'])
+        # wind_img = ee.Image(
+        #     ee.ImageCollection(input_coll.map(wind_magnitude)).mean())
+
+        ea_img = calcs._actual_vapor_pressure(
+            pair=calcs._air_pressure(elev, method),
+            q=input_coll.select(['SPFH']).mean())
+
+        return cls(
+            tmax=input_coll.select(['TMP']).max(),
+            tmin=input_coll.select(['TMP']).min(),
+            ea=ea_img,
+            rs=rs,
+            uz=input_coll.select(['WIND']).mean(),
+            zw=zw,
+            elev=elev,
+            lat=lat,
+            doy=ee.Number(start_date.getRelative('day', 'year')).add(1).double(),
+            method=method,
+            rso_type=rso_type,
+        )
